--[[----------------------------------------------------------------------------

MuyphotoExportServiceProvider.lua
Export service provider description for Lightroom Muyphoto uploader

--------------------------------------------------------------------------------

Copyright 2018 Grand Avocado Ltd.
All Rights Reserved.

------------------------------------------------------------------------------]]

	-- Lightroom SDK
local LrBinding = import 'LrBinding'
local LrDialogs = import 'LrDialogs'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrFunctionContext = import "LrFunctionContext"
local LrView = import 'LrView'
local LrHttp = import 'LrHttp'
local LrColor = import 'LrColor'

	-- Common shortcuts
local bind = LrView.bind
local share = LrView.share

	-- Include local components
require 'MuyphotoAPI'
require 'Utils'

	-- Initialize logger
local logger = import 'LrLogger'( 'MuyphotoLightroomPlugin' )
logger:enable("print")

	-- Publish/Export dialog settings
local exportServiceProvider = {
	hideSections = { 'exportLocation', 'video' },
	allowFileFormats = { 'JPEG' },
	allowColorSpaces = { 'sRGB' },
	hidePrintResolution = false,
	canExportVideo = false,
}

	-- Preset fields
exportServiceProvider.exportPresetFields = {
	{ key = 'loginStatus'			, default = "loggedOut" },
	{ key = 'loginError'			, default = nil },
	{ key = 'authToken'				, default = nil },
	{ key = 'loggedUserFullName'	, default = '' },
	{ key = 'loggedUserEmail'		, default = '' },
	{ key = 'loginDateTime'			, default = '' },
	{ key = 'userEvents'			, default = nil },
}

	-- Enable publishing services
--exportServiceProvider.supportsIncrementalPublish = true
exportServiceProvider.supportsIncrementalPublish = 'only'

	-- Show a nice logo for publisher
exportServiceProvider.small_icon = 'logo.png'

--------------------------------------------------------------------------------

--------------------------------------------------------------------------------

function exportServiceProvider.processRenderedPhotos( functionContext, exportContext )

	--logger:info('processRenderedPhotos')

	local exportSession = exportContext.exportSession
	local exportSettings = assert( exportContext.propertyTable )
	local isPublishOperation = exportSettings.LR_isExportForPublish

	-- Get the # of photos.
	
	local nPhotos = exportSession:countRenditions()

	-- Set progress title.
	
	local progressScope = exportContext:configureProgress {
						title = nPhotos > 1
									and LOC( "$$$/Export/Publish/Progress=Uploading ^1 photos to server", nPhotos )
									or LOC "$$$/Export/Publish/Progress/One=Uploading photo to server",
					}

	-- Save off uploaded photo IDs so we can take user to those photos later.
	
	local uploadedPhotoIds = {}
	local publishedCollection = exportContext.publishedCollection
	local publishedCollectionInfo = exportContext.publishedCollectionInfo
	local isDefaultCollection = publishedCollectionInfo.isDefaultCollection
	local remotePhotoIdsForRenditions = {}

	-- Look for a photoset id for this collection.

	local remoteCollectionId = publishedCollectionInfo.remoteId
	for i, rendition in exportContext.exportSession:renditions() do
		local remotePhotoId = rendition.publishedPhotoId
		remotePhotoIdsForRenditions[ rendition ] = remotePhotoId
	end
	
	-- Iterate through photo renditions.
	
	for i, rendition in exportContext:renditions { stopIfCanceled = true } do
	
		-- Update progress scope.
		
		progressScope:setPortionComplete( ( i - 1 ) / nPhotos )
		
		-- Get next photo.

		local photo = rendition.photo

		-- See if we previously uploaded this photo.

		local remotePhotoId = remotePhotoIdsForRenditions[ rendition ]
		
		if not rendition.wasSkipped then

			local success, pathOrMessage = rendition:waitForRender()
			
			-- Update progress scope again once we've got rendered photo.
			
			progressScope:setPortionComplete( ( i - 0.5 ) / nPhotos )
			
			-- Check for cancellation again after photo has been rendered.
			
			if progressScope:isCanceled() then break end
			
			if success then
	
				-- Build up common metadata for this photo.
				
				--[[
				local title = getFlickrTitle( photo, exportSettings, pathOrMessage )
		
				local description = photo:getFormattedMetadata( 'caption' )
				local keywordTags = photo:getFormattedMetadata( 'keywordTagsForExport' )
				
				local tags
				
				if keywordTags then

					tags = {}

					local keywordIter = string.gfind( keywordTags, "[^,]+" )

					for keyword in keywordIter do
					
						if string.sub( keyword, 1, 1 ) == ' ' then
							keyword = string.sub( keyword, 2, -1 )
						end
						
						if string.find( keyword, ' ' ) ~= nil then
							keyword = '"' .. keyword .. '"'
						end
						
						tags[ #tags + 1 ] = keyword

					end

				end
				]]
				
				-- Flickr will pick up LR keywords from XMP, so we don't need to merge them here.
				--[[
				local is_public = privacyToNumber[ exportSettings.privacy ]
				local is_friend = booleanToNumber( exportSettings.privacy_friends )
				local is_family = booleanToNumber( exportSettings.privacy_family )
				local safety_level = safetyToNumber[ exportSettings.safety ]
				local content_type = contentTypeToNumber[ exportSettings.type ]
				local hidden = exportSettings.hideFromPublic and 2 or 1
				]]
				
				-- Because it is common for Flickr users (even viewers) to add additional tags via
				-- the Flickr web site, so we should not remove extra keywords that do not correspond
				-- to keywords in Lightroom. In order to do so, we record the tags that we uploaded
				-- this time. Next time, we will compare the previous tags with these current tags.
				-- We use the difference between tag sets to determine if we should remove a tag (i.e.
				-- it was one we uploaded and is no longer present in Lightroom) or not (i.e. it was
				-- added by user on Flickr and never was present in Lightroom).
				--[[
				local previous_tags = photo:getPropertyForPlugin( _PLUGIN, 'previous_tags' ) 
	
				-- If on a free account and this photo already exists, delete it from Flickr.

				if flickrPhotoId and not exportSettings.isUserPro then

					FlickrAPI.deletePhoto( exportSettings, { photoId = flickrPhotoId, suppressError = true } )
					flickrPhotoId = nil

				end
				]]

				-- Upload or replace the photo.
				
				--local didReplace = not not flickrPhotoId
				
				-- Upload the photo to Kuvatilaus.fi. If the filename ends with "-Edit", tell the backend to strip it out so that photos edited in Photoshop overwrite the original photo
				remotePhotoId = MuyphotoAPI.uploadPhoto( exportSettings, {
										eventId         = remoteCollectionId,
										photoId         = remotePhotoId,
										filePath        = pathOrMessage,
										fileNameStrip   = "-Edit",
									} )

				photo.catalog:withPrivateWriteAccessDo( function()
										photo:setPropertyForPlugin( _PLUGIN, 'remote_uuid', remotePhotoId ) 
									end )

				--[[
				if didReplace then
				
					-- The replace call used by FlickrAPI.uploadPhoto ignores all of the metadata that is passed
					-- in above. We have to manually upload that info after the fact in this case.
					
					if exportSettings.titleRepublishBehavior == 'replace' then
						
						FlickrAPI.callRestMethod( exportSettings, {
												method = 'flickr.photos.setMeta',
												photo_id = flickrPhotoId,
												title = title or '',
												description = description or '',
											} )
											
					end
	
					FlickrAPI.callRestMethod( exportSettings, {
											method = 'flickr.photos.setPerms',
											photo_id = flickrPhotoId,
											is_public = is_public,
											is_friend = is_friend,
											is_family = is_family,
											perm_comment = 3, -- everybody
											perm_addmeta = 3, -- everybody
										} )
	
					FlickrAPI.callRestMethod( exportSettings, {
											method = 'flickr.photos.setSafetyLevel',
											photo_id = flickrPhotoId,
											safety_level = safety_level,
											hidden = (hidden == 2) and 1 or 0,
										} )
	
					FlickrAPI.callRestMethod( exportSettings, {
											method = 'flickr.photos.setContentType',
											photo_id = flickrPhotoId,
											content_type = content_type,
										} )
		
				end
	
				FlickrAPI.setImageTags( exportSettings, {
											photo_id = flickrPhotoId,
											tags = table.concat( tags, ',' ),
											previous_tags = previous_tags,
											is_public = is_public,
										} )
				]]

				-- When done with photo, delete temp file. There is a cleanup step that happens later,
				-- but this will help manage space in the event of a large upload.
				
				LrFileUtils.delete( pathOrMessage )
	
				-- Remember this in the list of photos we uploaded.
	
				uploadedPhotoIds[ #uploadedPhotoIds + 1 ] = remotePhotoId
				
				-- If this isn't the Photostream, set up the photoset.
				--[[
				if not photosetUrl then
	
					if not isDefaultCollection then
	
						-- Create or update this photoset.
	
						photosetId, photosetUrl = FlickrAPI.createOrUpdatePhotoset( exportSettings, {
													photosetId = photosetId,
													title = publishedCollectionInfo.name,
													--		description = ??,
													primary_photo_id = uploadedPhotoIds[ 1 ],
												} )
				
					else
	
						-- Photostream: find the URL.
	
						photosetUrl = FlickrAPI.constructPhotostreamURL( exportSettings )
	
					end
					
				end
				]]

				-- Record this Flickr ID with the photo so we know to replace instead of upload.

				rendition:recordPublishedPhotoId( remotePhotoId )
				
				--[[
				local photoUrl
							
				if ( not isDefaultCollection ) then
					
					photoUrl = FlickrAPI.constructPhotoURL( exportSettings, {	
											photo_id = flickrPhotoId,
											photosetId = photosetId,
											is_public = is_public,
										} )	
										
					-- Add the uploaded photos to the correct photoset.

					FlickrAPI.addPhotosToSet( exportSettings, {
									photoId = flickrPhotoId,
									photosetId = photosetId,
								} )
					
				else
					
					photoUrl = FlickrAPI.constructPhotoURL( exportSettings, {
											photo_id = flickrPhotoId,
											is_public = is_public,
										} )
										
				end
				rendition:recordPublishedPhotoUrl( photoUrl )
				]]
						
				-- Because it is common for Flickr users (even viewers) to add additional tags
				-- via the Flickr web site, so we can avoid removing those user-added tags that
				-- were never in Lightroom to begin with. See earlier comment.
				--[[
				photo.catalog:withPrivateWriteAccessDo( function()
										photo:setPropertyForPlugin( _PLUGIN, 'previous_tags', table.concat( tags, ',' ) ) 
									end )
				]]
			
			end
		
		else
		
			-- To get the skipped photo out of the to-republish bin.
			rendition:recordPublishedPhotoId( rendition.publishedPhotoId )
			
		end

	end
	
	--[[
	if #uploadedPhotoIds > 0 then
	
		if ( not isDefaultCollection ) then
			
			exportSession:recordRemoteCollectionId( remoteCollectionId )
					
		end
	
		-- Set up some additional metadata for this collection.
		exportSession:recordRemoteCollectionUrl( remoteCollectionUrl )
		
	end
	]]

	-- Organize the photos after everything has been uploaded
	MuyphotoAPI.organizePhotos( exportSettings, remoteCollectionId, uploadedPhotoIds )

	-- Progress is ready
	progressScope:done()
	
end

function exportServiceProvider.startDialog( propertyTable )
	if propertyTable.loginStatus == "loggedIn" and propertyTable.authToken ~= nil then
		LrFunctionContext.postAsyncTaskWithContext(
			LOC "$$$/Export/EventUpdateTaskName=Update event collection",
			function( context )
				MuyphotoAPI.getEvents( nil )
			end
		)
	else
		propertyTable.LR_cantExportBecause = LOC "$$$/Export/ExportDialog/Login/LoginRequiredTitle=You haven't signed in yet"
	end
end

function exportServiceProvider.endDialog( propertyTable )
	Utils.logInfo("exportServiceProvider.endDialog:propertyTable:", propertyTable)
	if propertyTable.loginStatus ~= "loggedIn" then
		LrFunctionContext.postAsyncTaskWithContext(
			LOC "$$$/Export/ExportDialog/Login/LoginAsyncTask=Signing in...",
			function( context )
				MuyphotoAPI.authLogin( propertyTable )
			end
		)
	elseif propertyTable.authToken ~= nil then
		LrFunctionContext.postAsyncTaskWithContext(
			LOC "$$$/Export/EventUpdateTaskName=Update event collection",
			function( context )
				MuyphotoAPI.getEvents( nil )
			end
		)
	end
end

--------------------------------------------------------------------------------
--- (optional) This plug-in defined callback function is called when the user 
 -- chooses this export service provider in the Export or Publish dialog. 
 -- It can create new sections that appear above all of the built-in sections 
 -- in the dialog (except for the Publish Service section in the Publish dialog, 
 -- which always appears at the very top).
 -- <p>Your plug-in's <a href="#exportServiceProvider.startDialog"><code>startDialog</code></a>
 -- function, if any, is called before this function is called.</p>
 -- <p>This is a blocking call. If you need to start a long-running task (such as
 -- network access), create a task using the <a href="LrTasks.html"><code>LrTasks</code></a>
 -- namespace.</p>
 -- <p>First supported in version 1.3 of the Lightroom SDK.</p>
	-- @param f (<a href="LrView.html#LrView.osFactory"><code>LrView.osFactory</code> object)
		-- A view factory object.
	-- @param propertyTable (table) An observable table that contains the most
		-- recent settings for your export or publish plug-in, including both
		-- settings that you have defined and Lightroom-defined export settings
	-- @return (table) An array of dialog sections (see example code for details)
	-- @name exportServiceProvider.sectionsForTopOfDialog
	-- @class function

function exportServiceProvider.sectionsForTopOfDialog( f, propertyTable )

	return {

		{
			title = LOC "$$$/Export/ExportDialog/Login/Title=Sign in to the service",
			
			synopsis = LOC "$$$/Export/ExportDialog/Login/SignIn=Sign in",

			f:column {

				place = "overlapping",
				margin = 3,

				f:view {

					visible = LrBinding.keyEquals( "loginStatus", "loggedOut" ),
					spacing = f:label_spacing(),
					margin = 3,

					f:row {
						f:static_text {
							title = LOC "$$$/Export/ExportDialog/Login/Email=E-mail:",
							alignment = "right",
							width = share "label_width", -- the shared binding
						},
						f:edit_field {
							value = bind "username_text",
							width_in_chars = 20,
						},
					},

					f:row {
						f:static_text {
							title = LOC "$$$/Export/ExportDialog/Login/Password=Password:",
							alignment = "right",
							width = share "label_width", -- the shared binding
						},
						f:password_field {
							value = bind "password_text",
							width_in_chars = 20,
						},
					},

					f:row {
						f:static_text {
							title = "",
							alignment = "right",
							width = share "label_width", -- the shared binding
						},
						f:push_button {
							title = LOC "$$$/Export/ExportDialog/Login/Synopsis=Sign in",
							enabled = true,
							alignment = 'center',
							action = function()
								LrFunctionContext.postAsyncTaskWithContext(
									LOC "$$$/Export/ExportDialog/Login/LoginAsyncTask=Signing in...",
									function( context )
										MuyphotoAPI.authLogin( propertyTable )
									end
								)
							end,
						},
						f:static_text {
							visible = LrBinding.keyIsNotNil("loginError"),
							title = bind "loginError",
							alignment = "left",
							text_color = LrColor 'red',
							width_in_chars = 40,
						},
					},
				},

				f:view {

					visible = LrBinding.keyEquals( "loginStatus", "loggingIn" ),
					spacing = f:label_spacing(),
					margin = 3,

					f:row {
						f:static_text {
							title = "",
							alignment = "right",
							width = share "label_width", -- the shared binding
						},
					},
					f:row {
						f:static_text {
							title = LOC "$$$/Export/ExportDialog/Login/LoginAsyncTask=Signing in...",
							alignment = "right",
							width = share "label_width", -- the shared binding
						},
						f:push_button {
							title = LOC "$$$/Export/ExportDialog/Login/Cancel=Cancel",
							enabled = true,
							alignment = 'center',
							action = function()
								MuyphotoAPI.authCancelLogin( propertyTable )
							end,
						},
					},
				},

				f:view {

					visible = LrBinding.keyEquals( "loginStatus", "loggedIn" ),
					spacing = f:label_spacing(),
					margin = 3,

					f:row {
						f:static_text {
							title = LOC "$$$/Export/ExportDialog/Login/Username=Signed in as:",
							alignment = "right",
							width = share "label_width", -- the shared binding
						},
						f:static_text {
							title = bind "loggedUserFullName",
							alignment = 'left',
						},
					},
					f:row {
						f:static_text {
							title = LOC "$$$/Export/ExportDialog/Login/Email=E-mail:",
							alignment = "right",
							width = share "label_width", -- the shared binding
						},
						f:static_text {
							title = bind "loggedUserEmail",
							alignment = 'left',
						},
					},
					--f:row {
					--	f:static_text {
					--		title = "Kirjautumisavain:",
					--		alignment = "right",
					--		width = share "label_width", -- the shared binding
					--	},
					--	f:static_text {
					--		title = bind "authToken",
					--		alignment = 'left',
					--	},
					--},
					f:row {
						f:static_text {
							title = "",
							alignment = "right",
							width = share "label_width", -- the shared binding
						},
						f:push_button {
							title = LOC "$$$/Export/ExportDialog/Login/SignOut=Sign out",
							enabled = true,
							alignment = 'center',
							action = function()
								MuyphotoAPI.authLogout( propertyTable )
							end,
						},
						f:push_button {
							title = LOC "$$$/Export/ExportMenuItems/UpdateEvents=Update events",
							enabled = true,
							alignment = 'center',
							action = function()
								LrFunctionContext.postAsyncTaskWithContext(
									LOC "$$$/Export/EventUpdateTaskName=Update event collection",
									function( context )
										LrDialogs.attachErrorDialogToFunctionContext(context)
										MuyphotoAPI.getEvents( propertyTable )
									end
								)
							end,
						},

					}
				},
			},
		},
	}

end

function exportServiceProvider.viewForCollectionSettings(f, publishSettings, info)
	return f:group_box {
		title = LOC "$$$/Export/CollectionSettingsTitle=Event information",
		size = 'medium',
		fill_horizontal = 1,
		bind_to_object = assert( info.collectionSettings ),

		f:row {
			f:static_text {
				title = LOC "$$$/Export/EventInfo/EventType=Event type:",
				alignment = "right",
				width = share "label_width", -- the shared binding
			},
			f:static_text {
				title = bind "eventTypeLabel",
				alignment = "left",
			},
		},

		f:row {
			f:static_text {
				title = LOC "$$$/Export/EventInfo/EventDate=Event date:",
				alignment = "right",
				width = share "label_width", -- the shared binding
			},
			f:static_text {
				title = bind "eventDateTime",
				alignment = "left",
			},
		}
	}
end

--------------------------------------------------------------------------------

return exportServiceProvider